#include <iostream>
#include <vector>
#include <algorithm>
#include <cmath>
#include <string>
#include <set>
#include <queue>

using namespace std;

using ll = long long;

vector<vector<int>> edges;
vector<vector<int>> rev_edges;
vector<pair<int, int>> score;
vector<int> marked;
vector<int> topSort;

void dfs(int node) {
	marked[node] = 1;
	for (int i = 0; i < edges[node].size(); ++i) {
		int next = edges[node][i];
		if (marked[next] == 0) {
			dfs(next);
		}
	}
	topSort.push_back(node + 1);
	marked[node] = 2;
}

void solve() {
	int n, m;
	std::cin >> n >> m;
	edges.assign(n, vector<int>());
	rev_edges.assign(n, vector<int>());
	score.assign(n, { 0, 0 });
	marked.assign(n, 0);
	topSort.clear();
	for (int i = 0; i < m; ++i) {
		int u, v;
		std::cin >> u >> v;
		--u;
		--v;
		score[v].first++;
		score[u].second++;
		edges[u].push_back(v);
		rev_edges[v].push_back(u);
	}
	priority_queue<pair<int, int>> pq;
	for (int i = 0; i < n; ++i) {
		pq.emplace(score[i].second - score[i].first, i);
	}

	while (!pq.empty()) {
		int node = pq.top().second;
		int node_score = pq.top().first;
		pq.pop();
		if (marked[node] != 0) {
			continue;
		}
		if (node_score != (score[node].second - score[node].first)) {
			continue;
		}
		marked[node] = 1;
		topSort.push_back(node + 1);
		for (auto next : edges[node]) {
			if (marked[next] != 0) {
				continue;
			}
			score[next].first--;
			pq.emplace(score[next].second - score[next].first, next);
		}
		for (auto prev : rev_edges[node]) {
			if (marked[prev] != 0) {
				continue;
			}
			score[prev].second--;
			pq.emplace(score[prev].second - score[prev].first, prev);
		}
	}

	for (auto x : topSort) {
		cout << x << ' ';
	}

	cout << endl;
}


int main() {
	std::ios_base::sync_with_stdio(false);
	std::cin.tie(0);
	std::cout.tie(0);
	int t;
	std::cin >> t;
	while (t-- > 0) {
		solve();
	}
}
